@@ -22,8 +22,9 @@
 #ifndef __PANGO_OT_H__
 #define __PANGO_OT_H__
 
-#include <freetype/freetype.h>
+#include <pango/pangofc-font.h>
 #include <pango/pango-glyph.h>
+#include <pango/pango-font.h>
 
 G_BEGIN_DECLS
 
@@ -32,6 +33,8 @@ G_BEGIN_DECLS
 typedef guint32  PangoOTTag;
 
 typedef struct _PangoOTInfo    PangoOTInfo;
+typedef struct _PangoOTBuffer  PangoOTBuffer;
+typedef struct _PangoOTGlyph   PangoOTGlyph;
 typedef struct _PangoOTRuleset PangoOTRuleset;
 
 typedef enum 
@@ -40,7 +43,22 @@ typedef enum
   PANGO_OT_TABLE_GPOS
 } PangoOTTableType;
 
-PangoOTInfo *pango_ot_info_new (FT_Face face);
+
+#define PANGO_OT_ALL_GLYPHS		((guint)0xFFFF)
+#define PANGO_OT_DEFAULT_LANGUAGE	((guint)0xFFFF)
+
+/* Note that this must match HB_GlyphItem */
+struct _PangoOTGlyph
+{
+  guint    glyph;
+  guint    properties;
+  guint    cluster;
+  gushort  component;
+  gushort  ligID;
+  gushort  property_cache;    /* Internal */
+};
+
+PangoOTInfo *pango_ot_info_get (FT_Face face);
 
 gboolean pango_ot_info_find_script   (PangoOTInfo      *info,
 				      PangoOTTableType  table_type,
@@ -71,56 +89,34 @@ PangoOTTag *pango_ot_info_list_features  (PangoOTInfo      *info,
 					  guint             script_index,
 					  guint             language_index);
 
-  /* A pointer to a function which loads a glyph.  Its parameters are
-   * the same as in a call to TT_Load_Glyph() -- if no glyph loading
-   * function will be registered with pango_ot_set_glyph_loader(),
-   * TT_Load_Glyph() will be called indeed.  The purpose of this function
-   * pointer is to provide a hook for caching glyph outlines and sbits
-   * (using the instance's generic pointer to hold the data).
-   *
-   * If for some reason no outline data is available (e.g. for an
-   * embedded bitmap glyph), _glyph->outline.n_points should be set to
-   * zero.  _glyph can be computed with
-   *
-   *    _glyph = HANDLE_Glyph( glyph )                                    
-   */
-typedef FT_Error  (*PangoOTGlyphLoader) (FT_Face      face,
-				         FT_UInt      glyphIndex,
-				         FT_Int       loadFlags,
-					 gpointer     data);
-
- /* A pointer to a function which selects the alternate glyph.  `pos' is
-  * the position of the glyph with index `glyphID', `num_alternates'
-  * gives the number of alternates in the `alternates' array.  `data'
-  * points to the user-defined structure specified during a call to
-  * TT_GSUB_Register_Alternate_Function().  The function must return an
-  * index into the `alternates' array.
-  */
-typedef FT_UShort  (*PangoOTAlternateFunc) (FT_ULong    pos,
-					    FT_UShort   glyphID,
-					    FT_UShort   num_alternates,
-					    FT_UShort*  alternates,
-					    gpointer    data);
+PangoOTBuffer *pango_ot_buffer_new        (PangoFcFont       *font);
+void           pango_ot_buffer_destroy    (PangoOTBuffer     *buffer);
+void           pango_ot_buffer_clear      (PangoOTBuffer     *buffer);
+void           pango_ot_buffer_set_rtl    (PangoOTBuffer     *buffer,
+					   gboolean           rtl);
+void           pango_ot_buffer_add_glyph  (PangoOTBuffer     *buffer,
+					   guint              glyph,
+					   guint              properties,
+					   guint              cluster);
+void           pango_ot_buffer_get_glyphs (PangoOTBuffer     *buffer,
+					   PangoOTGlyph     **glyphs,
+					   int               *n_glyphs);
+void           pango_ot_buffer_output     (PangoOTBuffer     *buffer,
+					   PangoGlyphString  *glyphs);
 
+void           pango_ot_buffer_set_zero_width_marks (PangoOTBuffer     *buffer,
+						     gboolean           zero_width_marks);
 
 PangoOTRuleset *pango_ot_ruleset_new (PangoOTInfo       *info);
 
-void pango_ot_ruleset_set_glyph_loader   (PangoOTRuleset       *ruleset,
-					  PangoOTGlyphLoader    func,
-					  gpointer              data,
-					  GDestroyNotify        notify);
-void pango_ot_ruleset_set_alternate_func (PangoOTRuleset       *ruleset,
-					  PangoOTAlternateFunc  func,
-					  gpointer              data,
-					  GDestroyNotify        notify);
-
 void            pango_ot_ruleset_add_feature (PangoOTRuleset   *ruleset,
 					      PangoOTTableType  table_type,
 					      guint             feature_index,
 					      gulong            property_bit);
-void            pango_ot_ruleset_shape       (PangoOTRuleset   *ruleset,
-					      PangoGlyphString *glyphs,
-					      gulong           *properties);
+void            pango_ot_ruleset_substitute  (PangoOTRuleset   *ruleset,
+					      PangoOTBuffer    *buffer);
+void            pango_ot_ruleset_position    (PangoOTRuleset   *ruleset,
+					      PangoOTBuffer    *buffer);
 
 #endif /* PANGO_ENABLE_ENGINE */
 
